"""Compatibility module for filter matching patch points."""

from ....search.content import ContentMatch, search_project_content_rg

__all__ = [
    "ContentMatch",
    "search_project_content_rg",
]
